# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301  USA

import quixote.http_request

from http_response import HTTPResponse
from quixote import get_session, get_publisher

class HTTPRequest(quixote.http_request.HTTPRequest):
    def __init__(self, *args, **kwargs):
        quixote.http_request.HTTPRequest.__init__(self, *args, **kwargs)
        self.response = HTTPResponse()

    _user = () # use empty tuple instead of None as None is a "valid" user value
    def get_user(self):
        if self._user != ():
            return self._user
        self._user = get_session().get_user()
        return self._user
    user = property(get_user)

    def get_server(self, clean = True):
        server_name = quixote.http_request.HTTPRequest.get_server(self)
        if clean and server_name.lower()[:7] in ('iframe.', 'iframe-'):
            server_name = server_name[7:]
        return server_name

    def get_local_url(self):
        '''Return the local part of the URL, query string included'''
        query = self.get_query()
        if query:
            return self.get_path() + '?' + query
        else:
            return self.get_path()

    def _process_multipart(self, length, params):
        # Make sure request.form doesn't contain unicode strings, converting
        # them all to strings in the site charset; it would contain unicode
        # strings when the user agent specifies a charset in a mime content
        # part, such a behaviour appears with some Nokia phones (6020, 6300)
        site_charset = get_publisher().site_charset
        # parse multipart data with the charset of the website
        if 'charset' not in params:
            params['charset'] = site_charset
        quixote.http_request.HTTPRequest._process_multipart(self, length, params)
        for k, v in self.form.items():
            if type(v) is unicode:
                self.form[k] = v.encode(site_charset)
