# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301  USA

import urllib2
import urllib


from quixote import get_publisher, redirect

from qommon import errors
from qommon import get_cfg

class SMS:
    """ This class allows to send a SMS using Mobyt provider """

    def __init__(self, provider = None):
        sms_cfg = get_cfg('sms', {})
        if not provider:
            self.provider = sms_cfg.get('mode', '')
        self.user = None
        self.password = None
        if self.provider == "mobyt":
            self.user = sms_cfg.get('mobyt_username', '')
            self.password = sms_cfg.get('mobyt_password', '')

    def _mobyt_send(self, sender, destinations, text, quality='l'):
        """ Send a sms using Mobyt provider"""
        rcpt = ""
        for dest in destinations:
            rcpt += "%s," % dest
        rcpt = rcpt[:len(rcpt) - 1]
        params = urllib.urlencode({
            'user': self.user,
            'pass': self.password,
            'rcpt': rcpt,
            'data': text,
            'sender': sender,
            'qty': quality
            })
        try:
            r = urllib2.urlopen("http://multilevel.mobyt.fr/sms/batch.php", params)
        except Exception, e:
            raise errors.SMSError("urlopen mobyt.fr failed : %s" % e)
        answer = r.read()
        r.close()
        if answer[:2] == "KO":
            raise errors.SMSError(answer[3:])

    def _mobyt_credit(self, type):
        """ """
        params = urllib.urlencode({
            'user': self.user,
            'pass': self.password,
            'type': type
            })
        try:
            r = urllib2.urlopen("http://multilevel.mobyt.fr/sms/credit.php", params)
        except Exception, e:
            raise errors.SMSError("urlopen mobyt.fr failed : %s" % e)
        answer = r.read()
        r.close()
        if answer[:2] == "KO":
            raise errors.SMSError(answer[3:])
        else:
            return answer[3:]
        

    def send(self, destinations, text, sender, quality=None):
        """
        Send a sms
        destinations : tuple with phone numbers (+YYXXXXXXXXX)
        """
        if self.provider == "mobyt":
            if not quality:
                quality = 'l'
            self._mobyt_send(sender, destinations, text, quality)
        else:
            raise errors.SMSError("Unknow SMS provider")

    def get_sms_left(self, type="standard"):
        """
        type (mobyt provider) : standard, lowcost or top
        """
        if self.provider == "mobyt":
            if type == "standard":
                return self._mobyt_credit("l")
            elif type == "lowcost":
                return self._mobyt_credit("ll")
            elif type == "top":
                return self._mobyt_credit("n")
            else:
                raise errors.SMSError("%s is invalid type for provider Mobyt" % type)
        else:
            raise errors.SMSError("Unknow SMS provider")

    def get_money_left(self):
        """ return money left in euros """
        if self.provider == "mobyt":
            return self._mobyt_credit("credit")
        else:
            raise errors.SMSError("Unknow SMS provider")
        
        
